// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KSANECORE_VERSION_H
#define KSANECORE_VERSION_H

#define KSANECORE_VERSION_STRING "26.04.1"
#define KSANECORE_VERSION_MAJOR 26
#define KSANECORE_VERSION_MINOR 4
#define KSANECORE_VERSION_PATCH 1
#define KSANECORE_VERSION ((26<<16)|(4<<8)|(1))

#endif
